//
//  PBViewController.m
//  CalculatorDemoNative
//
//  Created by Peter Bergström on 8/20/14.
//  Copyright (c) 2014 Peter Bergström. All rights reserved.
//

#import "PBViewController.h"

const NSInteger kMaxValueLength = 15;

const NSInteger kAllClear = 1;
const NSInteger kFlipSign = 2;
const NSInteger kModulus  = 3;

const NSInteger kNumberDec = 10;

const NSInteger kOpDivide = 1000;
const NSInteger kOpMult = 1001;
const NSInteger kOpSub = 1002;
const NSInteger kOpAdd = 1003;
const NSInteger kOpEquals = 1005;

const NSInteger OP_NONE = 0;

@interface PBViewController ()

@property (nonatomic, strong) NSString *aValue;
@property (nonatomic, strong) NSString *bValue;
@property (nonatomic, assign) BOOL isEnteringAValue;
@property (nonatomic, assign) NSInteger currentOperator;

@end

@implementation PBViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    [self reset];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


-(IBAction)performAction:(id)sender {
    UILabel *actionLabel = (UILabel *)sender;
    NSInteger action = [actionLabel tag];
    
    if(action == kAllClear) {
        [self reset];
    } else {
        if(self.isEnteringAValue) {
            self.aValue = [self performActionOnValue:action withValue: self.aValue];
        } else {
            self.bValue = [self performActionOnValue:action withValue: self.bValue];
        }
    }
    [self updateDisplay];
}

-(IBAction)pushNumber:(id)sender {
    UILabel *numberLabel = (UILabel *)sender;
    
    if (self.currentOperator != OP_NONE) {
        self.isEnteringAValue = NO;
    }
    
    NSInteger number = [numberLabel tag] % 100;
    NSString *numberString;
    
    if (number != kNumberDec) {
        numberString = [NSString stringWithFormat:@"%d", number];
    } else {
        numberString = @".";
    }

    
    if(self.isEnteringAValue) {
        NSMutableString *aValue = [NSMutableString stringWithString:self.aValue];
        if (aValue.length < kMaxValueLength) {
            [aValue appendFormat:@"%@", numberString];
        }
        self.aValue = aValue;
    } else {
        NSMutableString *bValue = [NSMutableString stringWithString:self.bValue];
        if (bValue.length < kMaxValueLength) {
            [bValue appendFormat:@"%@", numberString];
        }
        self.bValue = bValue;
    }
    [self updateDisplay];
}

-(IBAction)performOperation:(id)sender {
    UILabel *opLabel = (UILabel *)sender;
    NSInteger op = opLabel.tag;
    
    if (op == kOpEquals && self.currentOperator != OP_NONE) {
        if (self.isEnteringAValue) {
            self.aValue = [self performMathOperation:YES];
        } else {
            self.aValue =  [self performMathOperation:NO];
            self.isEnteringAValue = YES;
        }
        self.bValue = @"";
    } else {
        self.currentOperator = op;
    }
    [self updateDisplay];

}


-(NSString *)performMathOperation:(BOOL)isSingular {
    
    NSString *aValue = self.aValue;
    NSString *bValue = (isSingular) ? aValue : self.bValue;
    float a = [aValue floatValue];
    float b = [bValue floatValue];
    
    switch(self.currentOperator) {
        case kOpAdd:
            a = a + b;
            break;
        case kOpSub:
            a = a - b;
            break;
        case kOpMult:
            a = a * b;
            break;
        case kOpDivide:
            a = a / b;
            break;
    }
    
    return [[NSDecimalNumber decimalNumberWithString:[NSString stringWithFormat:@"%f", a]] stringValue];
}

-(NSString *)performActionOnValue:(NSInteger)action withValue:(NSString *)value {
    float ret = [value floatValue];
    
    if (action == kModulus) {
        ret = [value floatValue] / 100;
    } else if(action  == kFlipSign) {
        ret = [value floatValue] * -1;
    }
    
    return [[NSDecimalNumber decimalNumberWithString:[NSString stringWithFormat:@"%f", ret]] stringValue];
    
}


-(void)reset {
    self.aValue = @"";
    self.bValue = @"";
    self.isEnteringAValue = YES;
    self.currentOperator  = OP_NONE;
    [self updateDisplay];
}

-(void)updateDisplay {
    NSString *value;
    
    if (self.isEnteringAValue) {
        value = self.aValue;
    } else {
        value = self.bValue;
    }

    if (value.length == 0) {
        value = @"0";
    }
    
    self.displayNumber.text = value;
    
}
@end
